 // Author:      Brent Baccala 
 // Adopted for C++ by  Plyashkevich Viatcheslav <plyashkevich@yandex.ru>

 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
 // the Free Software Foundation; either version 2 of the License, or
 // (at your option) any later version.
 //--------------------------------------------------------------------------
 // All rights reserved. 

#if _RAW_HDLC_ == 0
#define _RAW_HDLC_ 1

// Hdlc packets pack and unpack, 
// for serial transmit and receive purposes.

#include "types_cpp.hpp"

typedef Types<sizeof(long int), sizeof(int), sizeof(short int), sizeof(char)>::Int32     INT32;
typedef Types<sizeof(long int), sizeof(int), sizeof(short int), sizeof(char)>::Uint32    UINT32;
typedef Types<sizeof(long int), sizeof(int), sizeof(short int), sizeof(char)>::Int16     INT16;
typedef Types<sizeof(long int), sizeof(int), sizeof(short int), sizeof(char)>::Uint16    UINT16;
typedef Types<sizeof(long int), sizeof(int), sizeof(short int), sizeof(char)>::Int8      INT8;
typedef Types<sizeof(long int), sizeof(int), sizeof(short int), sizeof(char)>::Uint8     UINT8;

class Hdlc
{
 struct StcAux;
 StcAux *pStcAux_;
	public:
	
		INT32 hdlcPack(UINT8 *src, UINT32 slen, UINT8 *dst, UINT32 dsize);
		INT32 hdlcUnPack(UINT8 *src, UINT32 slen, UINT8 *dst, UINT32 dsize);
                       
		Hdlc(UINT16 mode = 0);
		~Hdlc();
};                       

#endif

/*
           Description:
Hdlc object is used for packing of data in hdlc packets (in bit's stream)
and unpacking hdlc packets (bit's stream) to data. Format hdlc packets you may
find in a special literature, but you no need this information for successful
implementation of the object.
////////////////////////////////////////////////////////////
The method: INT32 hdlcPack(UINT8 *src, UINT32 slen, UINT8 *dst, UINT32 dsize) 
is used for creation of a hdlc packet. 
More detail:
src - address of a data's array (data that you need for a packing)
slen - data's size in bytes.
dst - addres of a bit's stream who created from data's array (address of a hdlc packet who will
be created after invoke this method).
dsize - space for bit's stream in bytes. You must to tell to the method, what buffer's space
you may to grant for creation hdlc packet. If you will grant very large buffer, you will waste a
memory (but a hdlc packet will be successful created), if you will grant very little buffer, 
the method can not place all bits of a stream and the method will return
a FAILURE (-1), so hdlc packet will not be created.
Recomendation: a dsize to have to be equal: ceil(6*slen/5 + 5) bytes.
Return values: 
Negative value(-1) - error: not sufficient space for creation of a hdlc packet.
Positive value - success, real size in bytes of a new created hdlc packet (may be less or equal dsize).
////////////////////////////////////////////////////////////
The method: INT32 hdlcUnPack(UINT8 *src, UINT32 slen, UINT8 *dst, UINT32 dsize)
is used for unpackin of a hdlc packet.
More detail:
src - address of a buffer who receives bit's stream (hdlc packet).
slen - a size of the src's buffer. slen may be less, equal or more then hdlc packet's length.
if slen is less then hdlc packet, this method must be invoked some times, while hdlc packet will not be received.
So hdlc packet is receives in parts and all parts are place in src buffer. 
dst - address of a space for data who been extract from hdlc packet.
dsize - space for data in bytes. The dsize must be large enough for all data, extracted from hdlc packet plus
2 bytes for checksum. If dsize is less then data, extracted from hdlc packet + 2 checksum's bytes, the method will
return a FAILURE (-1), and hdlc packet will be discarded.
Return values:
Negative value (-1) - error 1: not sufficient space for data, extracted from hdlc packet or for checksum. 
Negative value (-1)	- error 2: check sum failure(some data are damaged).
In both cases, a hdlc packet will be discarded.
0 - preambule finding or hdlc packet is received but still not completed (for "in parts" receive).
Positive value - success, size of data was been extracted from hdlc packet,
so hdlc packet was successful received and check sum was successful checked.
//////////////////////////////////////////////////////////////
Constructor: Hdlc(UINT16 mode)
mode == 0, sane frames.
mode == 1, by Siemens IOM-2 interface, B1 mode.
mode == 2, by Siemens IOM-2 interface, B2 mode.

example:
#include <stdio.h>
// simple loopback
UINT8 out[48], data[34];
INT32 result;
Hdlc hdlc;

	while(1)
	{
		result = hdlcPack(data, 32, out, 44);
		if(result < 0)
		{
			printf("transmit error\n");
			break;
		}
		
		result = hdlcUnPck(out, 16, data, 34);
		if(result < 0)
		{
			printf("receive error\n");
			break;
		}
		
		result = hdlcUnPck(&out[16], 16, data, 34);
		if(result < 0)
		{
			printf("receive error\n");
			break;
		}
		
		result = hdlcUnPck(&out[32], 16, data, 34);
		if(result < 0)
		{
			printf("receive error\n");
			break;
		}
		if(!result)
		{
			printf("I don't understand\n");
			break;
		}
		if(result > 0)
		{
			printf("receive was done, in data[32] and data[33] is checksum\n");
			break;			
		}
	}

real implementation of a packing:

UINT8 bits_out[313], data_out[256];
      
      while(1)
      {
      	hdlcPack(data_out, 256, bits_out, 313 // large enough dsize
      	);	
      	// after that, for examle, bits_out array will be transfer by DMA to serial port
      }
      
real implementation of an unpacking:
#include <stdio.h>
const UINT16 HDLC_SIZE = ...(you must to know max size of a hdlc frame in you system).
UINT8 bits_in[256], data_in[HDLC_SIZE];
INT32 result;
      
      while(1)
      {
        // bits_in, for example, is received from serial port by DMA.
      	result = hdlcUnPack(bits_in, 256, data_in, HDLC_SIZE);
      	if(!result)
      		continue;
      	if(result > 0)
      	{
      		for(int ii = 0; ii < result; ++ii)
      		{
      		//work with data_in	
      		}
      	}
      	if(result < 0)
      	{
      		printf("hdlc packet erroe\n");
      	}
      }
      
*/


